package Octothorpe;
use Moose;
sub xmlns {}
sub root_element { "Octothorpe" }
use PRANG::Graph;
# class tests mixed graph structure:
#   Seq -> Quant -> Element
#   Seq -> Element
has_element "hyphen" =>
	is => "ro",
	isa => "Bool",
	xml_nodeName => "emdash",
	predicate => "has_hyphen",
	;
has_element "colon" =>
	is => "ro",
	isa => "Str",
	;
has_element "apostrophe" =>
	is => "ro",
	isa => "Ampersand",
	xml_required => 0,
	;

has_element "pipe" =>
	is => "ro",
	isa => "Fingernails",
	xml_required => 0,
	;

has_element "section_mark" =>
	is => "ro",
	isa => "SectionMark",
	xml_required => 0,
	xmlns => "uri:type:A",
	;

has_element "curly_brackets" =>
	is => "ro",
	isa => "CurlyBrackets",
	xml_required => 0,
	;

with "PRANG::Graph", "PRANG::Graph::Class";

package Ampersand;
use Moose;
sub xmlns {}
use PRANG::Graph;
with "PRANG::Graph::Class";
# class tests: Quant -> Element
has_element "interpunct" =>
	is => "ro",
	isa => "Int",
	predicate => "has_interpunct",
	;

package Caret;
use Moose;
sub xmlns {}
use PRANG::Graph;
with "PRANG::Graph::Class";
# class tests:
#    Choice -> Element
#    Choice -> Element -> Text
has_element "solidus" =>
	is => "ro",
	isa => "Octothorpe|Int",
	xml_nodeName => {
		"braces" => "Int",
		"parens" => "Octothorpe",
	},
	;

package Asteriks;
use Moose;
sub xmlns {}
use PRANG::Graph;
with "PRANG::Graph::Class";
# class tests:
#    Quant -> Choice -> Element
#    Quant -> Choice -> Text
has_element "bullet" =>
	is => "ro",
	isa => "ArrayRef[Str|Int]",
	xml_max => 5,
	xml_nodeName => {
		"umlout" => "Int",
		"guillemets" => "Str",
	},
	;

package Pilcrow;
use Moose;
sub xmlns {}
use PRANG::Graph;
with "PRANG::Graph::Class";

#    Quant -> Element
has_element "backslash" =>
	is => "ro",
	isa => "ArrayRef[Asteriks]",
	xml_required => 0,
	;

package Deaeresis;
use Moose;
sub xmlns {}
use PRANG::Graph;
with "PRANG::Graph::Class";

#    Quant -> Choice with type/nodeName mapping
has_element "asterism" =>
	is => "ro",
	isa => "ArrayRef[Caret|Pilcrow|Str]",
	xml_min => 0,
	xml_nodeName => {
		"space" => "Caret",
		"underscore" => "Pilcrow",
		"slash" => "Str",
	},
	;

# test attribute name wildcarding
has_attr "currency" =>
	is => "ro",
	isa => "HashRef[Str]",
	xml_name => "*",
	;

# test attribute namespace wildcarding
has_attr "period" =>
	is => "ro",
	isa => "ArrayRef[Str]",
	xmlns => "*",
	xmlns_attr => "period_ns",
	;

has "period_ns" => is => "ro";

package Fingernails;
use Moose;
sub xmlns {}
use PRANG::Graph;
#    Class tests: Seq -> Element

# test attribute xmlns wildcarding
has_attr "currency" =>
	is => "ro",
	isa => "Str",
	xml_name => "dollar_sign",
	;
has 'currency_ns' =>
	is => "ro",
	;

has_element "fishhooks" =>
	is => "ro",
	isa => "Deaeresis",
	;

with "PRANG::Graph::Class";

package SectionMark;

use Moose;
use PRANG::Graph;
sub xmlns {}

# This class tests:
#     Seq -> Quant -> Choice -> Element
#      \                   `--> Element
#       \                  `--> Text
#        `-> Quant -> Choice -> Element
#                          `--> Element

# test mixed XML
has_element "double_angle_quotes" =>
	is => "ro",
	isa => "ArrayRef[Ampersand|Str|SectionMark]",
	xml_nodeName => {
		"" => "Str",
		"interrobang" => "Ampersand",
                "section_mark" => "SectionMark",
	},
	;

# test the "more element names than types" case - extra attribute
# required to record the node name.
has_element "percent_sign" =>
	is => "ro",
	isa => "Ampersand",
	xml_required => 0,
	xml_nodeName => {
		"degree" => "Ampersand",
		"period" => "Ampersand",
	},
	xml_nodeName_attr => "percent_sign_type",
	;

has "percent_sign_type" =>
	is => "ro",
	;

# test the "more types than element names" case in the docs (no extra
# attributes required, but namespaces vital)
has_element "broken_bar" =>
	is => "ro",
	isa => "Ampersand|Caret",
	xml_required => 0,
	xml_nodeName => {
		"trumpery:broken_bar" => "Ampersand",
		"rubble:broken_bar" => "Caret",
	},
	xml_nodeName_prefix => {
		"trumpery" => "uri:type:A",
		"rubble" => "uri:type:B",
	},
	;

has_attr "suspension_points" =>
	is => "ro",
	isa => "Str",
	xmlns => "uri:type:C",
	xml_required => 0,
	;

with "PRANG::Graph::Class";

package CurlyBrackets;

use Moose;
use PRANG::Graph;
sub xmlns {}

# test the "more element names than types" case - extra attribute
# required to record the node name.  This one should always go at
# the end of the class as it will happily eat all following elements
# (also should be kept in mind when trying to write invalid tests)
has_element "square_brackets" =>
	is => "ro",
	isa => "ArrayRef[Ampersand]",
	xml_required => 0,
	xml_nodeName => "*",
	xml_nodeName_attr => "square_brackets_type",
	;

has "square_brackets_type" =>
	is => "ro",
	;

with "PRANG::Graph::Class";

# Copyright (C) 2009, 2010  NZ Registry Services
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the Artistic License 2.0 or later.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Artistic License 2.0 for more details.
#
# You should have received a copy of the Artistic License the file
# COPYING.txt.  If not, see
# <http://www.perlfoundation.org/artistic_license_2_0>
